@@ -1,7 +1,7 @@
 """
 ``PluginManager`` unit and public API testing.
 """
-import sys
+import importlib.metadata
 from typing import Any
 from typing import List
 
@@ -13,11 +13,6 @@ from pluggy import HookspecMarker
 from pluggy import PluginManager
 from pluggy import PluginValidationError
 
-if sys.version_info >= (3, 8):
-    from importlib import metadata as importlib_metadata
-else:
-    import importlib_metadata
-
 
 hookspec = HookspecMarker("example")
 hookimpl = HookimplMarker("example")
@@ -156,6 +151,21 @@ def test_register_hookwrapper_not_a_generator_function(he_pm: PluginManager) ->
     assert excinfo.value.plugin is plugin
 
 
+def test_register_both_wrapper_and_hookwrapper(he_pm: PluginManager) -> None:
+    class hello:
+        @hookimpl(wrapper=True, hookwrapper=True)
+        def he_method1(self):
+            yield  # pragma: no cover
+
+    plugin = hello()
+
+    with pytest.raises(
+        PluginValidationError, match="wrapper.*hookwrapper.*mutually exclusive"
+    ) as excinfo:
+        he_pm.register(plugin)
+    assert excinfo.value.plugin is plugin
+
+
 def test_register(pm: PluginManager) -> None:
     class MyPlugin:
         @hookimpl
@@ -266,7 +276,7 @@ def test_historic_with_subset_hook_caller(pm: PluginManager) -> None:
 
 @pytest.mark.parametrize("result_callback", [True, False])
 def test_with_result_memorized(pm: PluginManager, result_callback: bool) -> None:
-    """Verify that ``_HookCaller._maybe_apply_history()`
+    """Verify that ``HookCaller._maybe_apply_history()`
     correctly applies the ``result_callback`` function, when provided,
     to the result from calling each newly registered hook.
     """
@@ -360,7 +370,7 @@ def test_register_historic_incompat_hookwrapper(pm: PluginManager) -> None:
         pm.register(Plugin())
 
 
-def test_register_historic_incompat_generator_fun(pm: PluginManager) -> None:
+def test_register_historic_incompat_wrapper(pm: PluginManager) -> None:
     class Hooks:
         @hookspec(historic=True)
         def he_method1(self, arg):
@@ -369,7 +379,7 @@ def test_register_historic_incompat_generator_fun(pm: PluginManager) -> None:
     pm.add_hookspecs(Hooks)
 
     class Plugin:
-        @hookimpl()
+        @hookimpl(wrapper=True)
         def he_method1(self, arg):
             yield
 
@@ -575,7 +585,7 @@ def test_load_setuptools_instantiation(monkeypatch, pm: PluginManager) -> None:
     def my_distributions():
         return (dist,)
 
-    monkeypatch.setattr(importlib_metadata, "distributions", my_distributions)
+    monkeypatch.setattr(importlib.metadata, "distributions", my_distributions)
     num = pm.load_setuptools_entrypoints("hello")
     assert num == 1
     plugin = pm.get_plugin("myname")
